#!/usr/bin/perl -w

package backup;

my $tmp_dir="/var/tmp/backup.$$";
my @files=(
	"/etc/network/interfaces",
	"/etc/hosts",
	"/etc/hostname",
	"/etc/apache2/apache2.conf",
	"/etc/apache2/conf.d/*",
	"/etc/resolv.conf",
	"/etc/ntp.conf",
	"/etc/bind/named.conf",
	"/etc/bind/zonefile",
	"/etc/dhcpd.conf",
	"/etc/mail/aliases",
	"/etc/asterisk/*",
	"/etc/zaptel.conf",
	"/tftpboot/spa*.cfg",
	"/tftpboot/aastra.cfg",
	"/tftpboot/init.cfg",
);
if( -f "/var/lib/asterisk/sounds/custom") {
  push @files,"/var/lib/asterisk/sounds/custom";
}
if( -f "/var/lib/asterisk/sounds/rec") {
  push @files,"/var/lib/asterisk/sounds/rec";
}

sub error($$) {
  my ( $msg, $num ) = @_;
  print "Content-type: text/html\n";
  print "\n";
  print "\n";
  print "<html><head><title>Error creating backup</title></head>\n";
  print "<body><h1>".$msg."</h1>\n";
  print "<p>Return code ".$num."</body>\n";

  unlink "$tmp_dir/website.tgz";
  unlink "$tmp_dir/system.tgz";
  unlink "$tmp_dir/pg.dump.gz";
  unlink "$tmp_dir/log.txt";
  rmdir $tmp_dir;

  exit 0;
}

mkdir $tmp_dir;

print "X-Keepalive: ping $$\n";
my $res=system("/usr/bin/pg_dumpall -Upostgres | gzip -9 >$tmp_dir/pg.dump.gz 2>$tmp_dir/log.txt");
error("Failed to dump PG database",$res) if( $res );

print "X-Keepalive: ping $$\n";
$res=system("/bin/tar czf $tmp_dir/website.tgz --exclude=\"/var/www/html/images/*\" --exclude=\"/var/www/html/icons/*\" --exclude=\"/var/www/html/backup/*\" --exclude=\"/var/www/html/docs/*\" --exclude=\"/var/www/html/*.bin\" --exclude=\"/var/www/html/*.exe\" /var/www/html/  2>>$tmp_dir/log.txt");
error("Failed to save website customisations",$res) if( $res );

print "X-Keepalive: ping $$\n";
$res=system("/bin/tar czf $tmp_dir/system.tgz ".(join " ",@files)."  2>>$tmp_dir/log.txt");
error("Failed to save system settings",$res) if( $res );

print "X-Keepalive: ping $$\n";
$res=system("cd $tmp_dir; /bin/tar czf $tmp_dir/backup.tgz website.tgz system.tgz pg.dump.gz log.txt 2>/dev/null");
error("Failed to create final backup file",$res) if( $res );

unlink "/var/www/html/backup/backup.tgz";
rename "$tmp_dir/backup.tgz", "/var/www/html/backup/backup.tgz";

print "Content-type: text/plain\n";
print "\nOK\n";

unlink "$tmp_dir/website.tgz";
unlink "$tmp_dir/system.tgz";
unlink "$tmp_dir/pg.dump.gz";
unlink "$tmp_dir/log.txt";
rmdir $tmp_dir;

exit 0;
