#!/usr/bin/perl -w

use strict;
use Digest::MD5;
use File::Copy;

print "Content-type: text/plain\r\n\r\n";

my %flist=();
my %dlist=();
my $do_it = 0;
my $will_do_it = 0;
my @do_list=();

our $cachedurl="";
our @badurl=();
our $tmpdir="/tmp/fw.$$";
mkdir $tmpdir;

if( defined $ARGV[0] && $ARGV[0] eq "--confirm" && defined $ARGV[1] && length($ARGV[1]) != 0 ) {
	$will_do_it = 1;
	@do_list = split(/\|/, $ARGV[1]);
}

if( !open XML, "</var/www/html/upgrade/firmware.xml" ) {
	print "FAIL Cannot open XML firmware list\n";
} else { 
	foreach my $pass ( (1,2) ) {
		if( $pass == 1 && $will_do_it == 0 ) {
			next;
		}
		foreach my $i ( glob "/tftpboot/*" ) {
			next unless( -f $i );
			$i =~ s@/tftpboot/@@;
			$flist{$i} = "exist";
			$dlist{$i} = "(other)";
		}
		if( $pass == 2 ) {
			$do_it = $will_do_it;
		}
		my $device = "";
		my $file = "";
		my $md5 = "";
		my $link = "";
		my @url=();
		my @mv=();
		while( <XML> ) {
			(my $dev = $device) =~ s/\(/\\(/;
			$dev =~ s/\)/\\)/;
			if( $_ =~ /<keep\s+name="(.*)"/ && defined $flist{$1} ) {
				$flist{$1} = "keep";
				next;
			}
			if( $_ =~ /<device\s+name="(.*)"/ ) {
				$device = $1;
				$file="";
				$md5="";
				$link="";
				@url=();
				@mv=();
				next;
			}
			if( $_ =~ /<rename\s+from="([^"]*)"\s*to="([^"]*)"\s*/ ) {
				push(@mv, { from => $1, to => $2 });
				next;
			}
			if( $_ =~ /<url\s+(?:index="([^"]*)")?\s*type="([^"]*)"\s*(?:notprovides="([^"]*)")?\s*(?:provides="([^"]*)")?\s*>(.*?)</ ) {
				if( defined($2) && $5 ne "" ) {
					push(@url, { index => $1, type => $2, notprovides => $3, provides => $4, url => $5 });
	# print "{ index => ".($1||"").", type => ".($2||"").", notprovides => ".($3||"").", provides => ".($4||"").", url => ".($5||"")." }\n";
				}
				next;
			}
			if( $_ =~ /<file\s+md5sum="(.*)"\s+name="(.*)"/ ) {
				$md5=$1;
				$file=$2;
				$link="";
				$dlist{$file}=$device;
				if( ! -f "/tftpboot/".$file ) {
					$flist{$file} = "missing";
					if( $do_it && grep(/^${dev}$/,@do_list) == 1 && &fetch_it($file, $md5, \@url, \@mv) ) {
						$flist{$file} .= " (fixed)";
					}
					next;
				}
				my $ctx = Digest::MD5->new;
				if( open FILE, "<", "/tftpboot/".$file ) {
					$ctx->addfile(*FILE);
					my $digest = $ctx->hexdigest;
					close FILE;
					if( $md5 ne $digest ) {
						$flist{$file} = "md5 ".$digest;
						if( $do_it && grep(/^${dev}$/,@do_list) == 1 && &fetch_it($file, $md5, \@url, \@mv) ) {
							$flist{$file} .= " (fixed)";
						}
						next;
					}
				} else {
					$flist{$file} = "md5 fail";
					next;
				}
				$flist{$file} = "ok";
				next;
			}
			if( $_ =~ /<link\s+name="(.*)"/ ) {
				$link="$1";
				$dlist{$link}=$device;
				if( ! -f "/tftpboot/".$link ) {
					$flist{$link} = "missing";
					if( $do_it && grep(/^${dev}$/,@do_list) == 1 && -f "/tftpboot/".$file ) {
						unlink "/tftpboot/".$link;
						link "/tftpboot/".$file, "/tftpboot/".$link;
						$flist{$link} .= " (fixed)";
					}
					next;
				}
				my @inum1 = stat("/tftpboot/".$link);
				if( $inum1[3] < 2 ) {
					$flist{$link} = "not link";
					if( $do_it && grep(/^${dev}$/,@do_list) == 1 && -f "/tftpboot/".$file ) {
						unlink "/tftpboot/".$link;
						link "/tftpboot/".$file, "/tftpboot/".$link;
						$flist{$link} .= " (fixed)";
					}
					next;
				}

				if( ! -f "/tftpboot/".$file ) {
					$flist{$link} = "wrong link";
					if( $do_it && grep(/^${dev}$/,@do_list) == 1 && -f "/tftpboot/".$file ) {
						unlink "/tftpboot/".$link;
						link "/tftpboot/".$file, "/tftpboot/".$link;
						$flist{$link} .= " (fixed)";
					}
					next;
				}
				my @inum2 = stat("/tftpboot/".$file);
				if( $inum1[1] != $inum2[1] ) {
					$flist{$link} = "wrong link";
					if( $do_it && grep(/^${dev}$/,@do_list) == 1 ) {
						unlink "/tftpboot/".$link;
						link "/tftpboot/".$file, "/tftpboot/".$link;
						$flist{$link} .= " (fixed)";
					}
					next;
				}

				$flist{$link} = "ok";
				next;
			}
		}
		seek XML, 0, 0;
		if( $pass == 2 ) {
			print "OK 1.0.0\n";
		}
		foreach my $i ( sort keys %flist ) {
			$dlist{$i} ||= "";
			(my $dev = $dlist{$i}) =~ s/\(/\\(/;
			$dev =~ s/\)/\\)/;
			if( $flist{$i} eq "exist" && $do_it && grep(/^${dev}$/,@do_list) == 1 ) {
				unlink "/tftpboot/".$i;
				$flist{$i} .= " (fixed)";
			}
			if( $pass == 2 ) {
				print $dlist{$i}."|".$i." => ".$flist{$i}."\n";
			}
		}
	}
	close XML;
}
# system("ls -lR ".$tmpdir."; md5sum ".$tmpdir."/*; rm -rf ".$tmpdir);
# system("ls -lR ".$tmpdir."; rm -rf ".$tmpdir);
system("rm -rf ".$tmpdir);
exit 0;

sub do_move($) {
	my $mvref = shift;
	my @mv = @$mvref;
	while( $#mv >= 0 ) {
		my $href = shift(@mv);
		if( -f $tmpdir."/".$href->{from} && ! -f $tmpdir."/".$href->{to} ) {
			unlink $tmpdir."/".$href->{to};
			link $tmpdir."/".$href->{from}, $tmpdir."/".$href->{to};
			unlink $tmpdir."/".$href->{from};
		}
	}
}

sub fetch_it($$$$) {
	my $file = shift;
	my $md5 = shift;
	my $urlref = shift;
	my $mvref = shift;
	my @url = ();	# Holds a COPY of the URL list
print "Called fetch_it\n";
	foreach my $u ( @$urlref ) {
		my %tmp = %$u;
		push @url, \%tmp;
	}
	my $notdone = 1;
	my $onindex = 1;	# increased if failure occurs
	URL: while( $notdone && $#url >= 0 ) {
		my $href = shift(@url);
		$href->{index} ||= 1;
		$href->{provides} ||= "";
		$href->{notprovides} ||= "";
		(my $url_gets = $href->{url}) =~ s/^.*\/(.*?)$/$1/;
		if( $href->{provides} eq "" && $href->{type} eq "raw" ) {
			$href->{provides} = $url_gets;
		}
		if( $href->{type} eq "path" ) {
			$href->{provides} = $file;
			$href->{url} .= $file;
			$href->{type} = "raw";
		}
		my @prov = split(/,/, $href->{provides});
		my @nprov = split(/,/, $href->{notprovides});

		next if( $href->{index} != $onindex );		# we've moved to a retry
		next if( length($href->{provides}) > 0 && grep (/^${file}$/, @prov) == 0 );	# provided by this URL, but not matching this file
		next if( length($href->{notprovides}) > 0 && grep (/^${file}$/, @nprov) > 0 );	# not provided by this URL

		my $use_cache=0;
print "Want URL : ".$href->{url}."\n";
		for my $i ( @badurl ) {
			if( $i eq $href->{url} ) {
print "Bad URL : ".$href->{url}."\n";
				$onindex++;
				next URL;
			}
		}
print "Cached URL : ".$cachedurl."\n";
		if( $cachedurl ne $href->{url} ) {
			$cachedurl = "";
			system("rm -rf $tmpdir/*");
print "Fetching : ".$href->{url}."\n";
			my $res = system("cd ".$tmpdir." && wget --tries=1 ".$href->{url}." 2>/dev/null");
			$res <<= 8;
			if( $res != 0 ) {
print "Failed.\n";
				push(@badurl, $href->{url});
				$onindex++;
				next;
			}
			$cachedurl = $href->{url};
		} else {
			$use_cache=1;
		}
		my $ctx = Digest::MD5->new;
print "Type = ".$href->{type}."\n";
		if( $href->{type} eq "raw" ) {
			if( $use_cache == 0 ) {
				do_move($mvref);
				if( ! -f $tmpdir."/".$file ) {
print "Where's my file? Failed.\n";
					$onindex++;
					next;
				}
				my $digest="";
				if( open FILE, "<", $tmpdir."/".$file ) {
					$ctx->addfile(*FILE);
					$digest = $ctx->hexdigest;
print "New file has md5 = $digest\n";
					close FILE;
				}
				if( $digest ne $md5 ) {
print "What md5? $digest ne $md5 Failed.\n";
					$onindex++;
					next;
				}
			}
			unlink "/tftpboot/".$file;
			copy($tmpdir."/".$file, "/tftpboot/".$file);
			$notdone = 0;
			last;
		}
		if( $href->{type} eq "zip" ) {
			if( ! $use_cache ) {
				if( ! -f $tmpdir."/".$url_gets ) {
print "Where's my ZIP file? Failed.\n";
					$onindex++;
					next;
				}
				my $res = system("cd $tmpdir && unzip -d $tmpdir $tmpdir/$url_gets >/dev/null");
				unlink $tmpdir."/".$url_gets;
				$res <<= 8;
				if( $res != 0 ) {
print " unZIP Failed.\n";
					$onindex++;
					next;
				}
				do_move($mvref);
			}
			if( ! -f $tmpdir."/".$file ) {
print "Where's my unZIPped file? Failed.\n";
				$onindex++;
				next;
			}
			my $digest="";
			if( open FILE, "<", $tmpdir."/".$file ) {
				$ctx->addfile(*FILE);
				$digest = $ctx->hexdigest;
				close FILE;
			}
			if( $digest ne $md5 ) {
print "What unZIP md5? $digest ne $md5 Failed.\n";
				$onindex++;
				next;
			}
			unlink "/tftpboot/".$file;
			copy($tmpdir."/".$file, "/tftpboot/".$file);
			$notdone = 0;
			last;
		}
	}
	return 0 if( $notdone );
	return 1;
}

