#!/bin/sh

if [ `basename $0` = "restore.sh" ]; then
  KEEPIP=no
else
  if [ `basename $0` = "restore_keep.sh" ]; then
    KEEPIP=yes
  else
    echo "FAIL -30 Called as $0 - not recognised"
    exit -30
  fi
fi

function do_exit() {
  rm -rf /var/tmp/rest.$$
  rm -f ${SRC} ${SRC2} ${SRC2}.gz ${SRC3}
  if [ $1 -eq 0 ]
  then
    echo "OK"
  else
    echo "FAIL $1 $2"
  fi
  exit $1
}

echo "Content-Type: text/plain
"

exec 6>/dev/stdout
CONDIT=" WHERE attrib NOT LIKE 'my_%' AND attrib NOT LIKE 'sec_%' "

mkdir -p /var/tmp/rest.$$
SRC=/var/www/html/backup/restore.tar.gz
SRC2=/var/tmp/rest.$$/pg.dump
SRC3=/var/tmp/rest.$$/system.tgz
TMP1=/var/tmp/rest.$$/tmp.out
TMP2=/var/tmp/rest.$$/tmp.sql
[ ! -f "$SRC" ] && do_exit -1 "File to restore from is missing"
[ ! -s "$SRC" ] && do_exit -2 "File to restore from is empty"
rm -f ${SRC2} ${SRC2}.gz ${SRC3}
tar xzf $SRC -C /var/tmp/rest.$$/ pg.dump.gz system.tgz >/dev/null 2>&1
if [ $? -ne 0 ]; then
  do_exit -3 "Cannot extract from backup file"
fi
echo "X-Keepalive1: 1
"
gunzip -t ${SRC2}.gz >/dev/null 2>&1
if [ $? -ne 0 ]; then
  do_exit -4 "Database dump file uncompress error"
fi
echo "X-Keepalive2: 2
"
[ ! -f "$SRC3" ] && do_exit -5 "System files are missing"
[ ! -s "$SRC3" ] && do_exit -6 "System files are empty"

tar xzf $SRC3 -C /var/tmp/rest.$$/ etc/sysconfig/network etc/sysconfig/network-scripts/ifcfg-eth0 etc/network/interfaces >/dev/null 2>&1
if [ ! -d /var/tmp/rest.$$/etc ]; then
  do_exit -5 "Cannot extract from system-info file"
fi
if [ ! -f /var/tmp/rest.$$/etc/sysconfig/network-scripts/ifcfg-eth0 -a ! -f /var/tmp/rest.$$/etc/network/interfaces ]
then
  do_exit -7 "Cannot find interface info in system file"
fi

echo >$TMP2
ETH0=/var/tmp/rest.$$/etc/sysconfig/network-scripts/ifcfg-eth0
if [ -f $ETH0 ]
then
  echo "DELETE FROM settings WHERE attrib IN ('my_ip','my_gw','my_mask','my_net','my_bcast');" >> $TMP2
  IP=`grep IPADDR $ETH0 | head -1 | cut -f2 -d=`
  echo "INSERT INTO settings(attrib,value) VALUES('my_ip','${IP}');" >> $TMP2
  GW=`grep GATEWAY $ETH0 | head -1 | cut -f2 -d=`
  echo "INSERT INTO settings(attrib,value) VALUES('my_gw','${GW}');" >> $TMP2
  NM=`grep NETMASK $ETH0 | head -1 | cut -f2 -d=`
  echo "INSERT INTO settings(attrib,value) VALUES('my_mask','${NM}');" >> $TMP2
fi

echo "X-Keepalive3: 3
"
(
echo "START TRANSACTION;
DELETE FROM contact_att;
DELETE FROM contacts;" ;
for i in contacts contact_att; do
  zcat ${SRC2}.gz | sed -n "/^COPY $i .*\$/,/^\\\.\$/p"
  echo "X-Keepalive3${i}: ${i}
" >&6
done
echo "SELECT setval('contacts_contact_id_seq',MAX(contact_id)+100) FROM contacts; \
COMMIT;"
) | psql -Upostgres whitebeam >/dev/null 2>&1
RES=$(( $RES + $? ))
echo "X-Keepalive4: 4
"

echo "X-Keepalive5: 5
"
(
echo "START TRANSACTION;
DELETE FROM xlate;
DELETE FROM inbound_nos;
DELETE FROM contact_phone;
DELETE FROM contact_group;
DELETE FROM exten_contact;
DELETE FROM barred;
DELETE FROM rate_pfx;
DELETE FROM rate_res;
DELETE FROM rate;
DELETE FROM cdr;
DELETE FROM call_group;
DELETE FROM contact;
DELETE FROM ivr;
DELETE FROM ivr_menu;
DELETE FROM phone;
DELETE FROM phone_type;
DELETE FROM provider;
DELETE FROM routing;
DELETE FROM virt_exten;
DELETE FROM pbook;
DELETE FROM pbook_src;"
for i in virt_exten routing provider phone ivr_menu ivr contact call_group barred exten_contact contact_group contact_phone inbound_nos rate rate_res rate_pfx cdr pbook_src pbook xlate; do
  zcat ${SRC2}.gz | sed -n "/^COPY $i .*\$/,/^\\\.\$/p"
  echo "X-Keepalive5${i}: ${i}
" >&6
done
for i in settings; do
  if [ "$KEEPIP" = "yes" ]; then
    echo "DELETE FROM settings $CONDIT;"
    zcat ${SRC2}.gz | sed -n "/^COPY $i .*\$/,/^\\\.\$/p" | grep -v -e "^my_" -e "^sec_"
    echo "X-Keepalive5${i}: ${i}
" >&6
  else
    echo "DELETE FROM settings;"
    zcat ${SRC2}.gz | sed -n "/^COPY $i .*\$/,/^\\\.\$/p"
    echo "X-Keepalive5${i}: ${i}
" >&6
    cat $TMP2
  fi
done
echo "SELECT setval('cdr_acctid_seq',MAX(acctid)+500) FROM cdr; \
COMMIT;"
) | psql -Upostgres asterisk >$TMP1 2>&1
RES=$(( $RES + `grep -c ERROR $TMP1` ))
echo "X-Keepalive6: 6
"
psql -Upostgres asterisk </var/www/html/schema/schema_ast_data.sql >>$TMP1 2>&1
RES=$(( $RES + $? ))
echo "X-Keepalive8: 8
"

# Just to be sure...
wget -O- --tries=1 http://localhost/initialise.whtm 2>/dev/null >/var/tmp/initialise.log
# Flag change is called in the above page...

echo "
"
if [ $RES -eq 0 ]
then
	do_exit $RES;
else
	do_exit $RES "Database restore failed"
fi

