#!/usr/bin/perl -w

use strict;

print "Content-type: text/plain\r\n\r\n";

my %stat=();
my %fitness=();
my %size=();

my @up=glob("/boot/upgrade-*");
@up = grep (!/\.md5/, @up);

my @fi=glob("/boot/firmware-*");
@fi = grep (!/\.md5/, @fi);

my @sc=glob("/boot/upgrade_*.pl");
@sc = grep (!/\.md5/, @sc);

$stat{files} = [ @up, @fi, @sc ];

my $progress=`cat /boot/.download 2>/dev/null`;
my $result=`cat /boot/.result 2>/dev/null`;
my $current=`cat /.pabx/pabx 2>/dev/null`;
if( $current =~ /.*ROOTIMG=pabx-(.+)\.img/ ) {
	$current = $1;
} else {
	$current = "Unknown";
}
$stat{current}=[$current];
for my $f ( @up, @fi, @sc ) {
	my $problem="";
	my $sz=0;
	if( -f $f ) {
		$sz = (stat $f)[7];
	}
	if( $progress ne "" ) {
		# no md5 if downloading...
	} elsif( ! -f "$f.md5" ) {
		$problem="missing MD5 checksum file";
	} else {
		my $md5file=`cat $f.md5`;
		my $md5sum=`md5sum $f`;
		if( $md5file ne $md5sum ) {
			$problem="incorrect MD5 checksum !";
		}
	}

	my $ver=undef;
	my $ftype="";
	my $required=0;
	if( $f =~ /^\/boot\/upgrade-(.*)$/ ) { $ver=$1; $ftype="Upgrade image"; $required=1; }
	if( $f =~ /^\/boot\/firmware-(.*)$/ ) { $ver=$1; $ftype="Phone firmware"; }
	if( $f =~ /^\/boot\/upgrade_(.*).pl$/ ) { $ver=$1; $ftype="Upgrade program"; $required=1; }
	if( defined $ver ) {
		$fitness{$ver} ||= 0;
		$size{$ver} ||= 0;
		$stat{$ver} ||= ();
		if( $problem eq "" ) {
			$problem="downloaded OK";
			$fitness{$ver} += $required;
			$size{$ver} += $sz;
		} else {
			$fitness{$ver} = -99;
			$size{$ver} += $sz;
		}
		push(@{$stat{$ver}}, $ftype." ".$problem);
	}
}
if( length($progress) ) {
	$size{$progress} ||= 0;
	$stat{$progress}=["Download in progress", "Approx. ".int($size{$progress}/1024/1024)."MB downloaded"];
	$fitness{$progress}=-99;
	$stat{"busy"}=["$progress"];
}
$result =~ s/>/&gt;/g;
$result =~ s/</&lt;/g;
$result =~ s/\n/<br>/g;
$result =~ s/\|/,/g;
$stat{"result"}=["$result"];
for my $k ( sort keys %stat ) {
	print $k."|";
	print $fitness{$k} if( defined $fitness{$k} );
	print "|".join(":",@{$stat{$k}});
	print "\n";
}
