#!/usr/bin/perl -w
#
# (c) 2005-2008 IP Cortex Ltd.
# All rights reserved.  Unauthorised copying is not permitted.
# 

use IO::Handle;
use IO::Socket;
use IO::Select;
use Socket;
use Fcntl qw(:DEFAULT :flock);
use strict;

my $pid;
if( $pid = fork() ) {
  # I'm the parent
  exit(0);
} elsif( !defined $pid ) {
  # Error condition
  exit(0);
}
# I am a child...

my $message="";
my $socket;

# We hold a lock in /var/run/mon_ast.lock to be sure we are only running once.
my $lockfile;
open $lockfile,"> /var/run/mon_ast.lock" or exit(0);
unless( flock $lockfile, LOCK_EX|LOCK_NB ) {
  close $lockfile;
  exit(0);
}
print $lockfile $$;
$lockfile->autoflush(0);

open STDOUT,">/dev/null";
open STDERR,">/dev/null";;
open STDIN,"</dev/null";;

sub check_error() {
	return "Asterisk stopped intentionally\n" if( ! -f "/var/run/asterisk/asterisk.pid" );	#/* Asterisk stopped on purpose */

	open PID, "< /var/run/asterisk/asterisk.pid" or return "Cannot open PID file!\n";
	my $pid = <PID>;
	close PID;

	chomp $pid;
	$message.="Got PID of >$pid<\n";

	open PS, "ps axwww |" or return "Cannot open PS command!\n";
	while( <PS> ) {
		next unless( $_ =~ /^\s*${pid}\s+.*\/usr\/sbin\/asterisk.*$/ );
		return "No action taken - Asterisk still running!?\n";
	}
	close PS;
	$message.="Possible error condition - attempting restart\n";
	unlink "/var/run/asterisk/asterisk.pid";
	system("/etc/init.d/asterisk restart");
	return "Restart completed.\n";
}

for(;;) {
  if( $message ne "" ) {
    if( open SENDMAIL, "| /usr/sbin/sendmail -t" ) {
      print SENDMAIL "To: root\n";
      print SENDMAIL "Subject: monitor output from ".`hostname`."\n\n";

      print SENDMAIL "$message";
      print SENDMAIL "\n";
      close SENDMAIL;
    }
  }
  $message = "";

  select undef, undef, undef, 15;
  next if( ! -f "/var/run/asterisk/asterisk.pid" );		#/* Asterisk stopped on purpose */

  unless( $socket = IO::Socket::INET->new(PeerAddr	=> '127.0.0.1',
				     PeerPort	=> 5037,
				     Proto	=> 'tcp',
				     Type	=> SOCK_STREAM) ) {
	$message .= "Connect attempt failed @ ".`date`."\n";
	$message .= check_error();
  } else {
	$message .= "Connected @ ".`date`."\n";
	while( <$socket> ) {}
  	close $socket;
	$message .= "Socket closed @ ".`date`."\n";
	$message .= check_error();
  }
}
