#!/usr/bin/perl -w
#
# (c) 2005-2008 IP Cortex Ltd.
# All rights reserved.  Unauthorised copying is not permitted.
# 

use DBI;
use DBD::Pg;
use File::Copy;
use IO::Handle;
use IO::File;
use IO::Select;
use Socket;
use Fcntl qw(:DEFAULT :flock);
use strict;

my $pid;
if( $pid = fork() ) {
  # I'm the parent
  exit(0);
} elsif( !defined $pid ) {
  # Error condition
  exit(0);
}
# I am a child...

close STDIN;
close STDOUT;
close STDERR;

my $message="";
my $mail_delta=0;

# We hold a lock in /var/run/upd_ast.lock to be sure we are only running once.
my $lockfile;
open $lockfile,"> /var/run/upd_ast.lock" or exit(0);
unless( flock $lockfile, LOCK_EX|LOCK_NB ) {
  close $lockfile;
  exit(0);
}
print $lockfile $$;
$lockfile->autoflush(0);

unlink '/var/www/html/config/upd_ast.pipe';
system('mknod','/var/www/html/config/upd_ast.pipe','p') && die "Cannot create /var/run/upd_ast.pipe: $!";
chmod 0666, '/var/www/html/config/upd_ast.pipe';

my $datasink;
my $readset   = new IO::Select();
my $pipe = new IO::File( '/var/www/html/config/upd_ast.pipe',  O_RDWR|O_NONBLOCK );
$readset->add($pipe);

for(;;) {
  if( $message ne "" ) {
    if( $mail_delta != 0 && open SENDMAIL, "| /usr/sbin/sendmail -t" ) {
      print SENDMAIL "To: root\n";
      print SENDMAIL "Subject: update_asterisk.pl output\n\n";

      print SENDMAIL "$message";
      print SENDMAIL "\n";
      close SENDMAIL;
    }

    # Most frequest change fixed at 30 seconds after last one completes.
    select undef,undef,undef,30;
  }
  $message = "";
  my @ready = $readset->can_read(60);

  select undef,undef,undef,1;
  sysread $pipe, $datasink, 9999;

#  if( $#ready == -1 ) {
#    $message .= "TIMEOUT PING!\n";
#  } else {
#    $message .= "GOT PUNT!\n";
#  }
#
  my $db="asterisk";
  my $user="asterisk";
  my $pass="asterisk";

  our $dbh = DBI->connect( "dbi:Pg:dbname=$db;host=127.0.0.1", "$user", "$pass" ) or die "Cannot connect to database!";

  our $base="/var/www/html/config/configs";
  our $diff;
  our $res;

#
# Asterisk may update voicemail.conf.
# Collect these changes and push back to database
#
  my $count_updates=0;
  my @updates=();
  $diff=`diff -Nu $base/voicemail.conf.sent /etc/asterisk/voicemail.conf 2>/dev/null`;
  if( -f "$base/voicemail.conf.sent" && length($diff) > 0 ) {
    $message .= "Importing updated PIN numbers\n";
    foreach my $line ( split "\n",$diff ) {
      if( $line =~ /^\+([^\+']+?)\s*=>\s*([^,']+),.*$/ ) {
        $message .= "Vmail $1 -> $2\n";
        $count_updates++;
        $line =~ s/^\+(.+)$/$1/;
        push @updates, $line;
      }
    }
  }
  open VM,"< /etc/asterisk/voicemail.conf" or die "Cannot open /etc/asterisk/voicemail.conf!";
  my $section="";
  while ( $#updates >= 0 && (my $line=<VM>) ) {
    chomp $line;
    if( $line =~ /^\[(.+)\]$/ ) {
      $section = "$1";
      next;
    }
    if( $section eq "" ) {
      next;
    }
    if( $line eq $updates[0] ) {
      shift @updates;
      if( $line =~ /^([^\+']+?)\s*=>\s*([^,']+),.*$/ ) {
        $message .= "Vmail $section,$1 -> $2 ";
        my $vm="$1";
        my $pc="$2";
        my $sqlcmd = "UPDATE contact SET vmpin=? WHERE owned_extension=?;";
        my $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
        my $rc = $sth->execute($pc,$vm) or die "Cannot execute $sqlcmd!";
        $sth->finish();

# Same as DB.flag_change() in JS
        $dbh->do("UPDATE settings SET value=(EXTRACT(EPOCH FROM NOW())*1000)::bigint WHERE attrib='flag_change_date'");

        if( $rc > 0 ) {
          $message .= "User PIN\n";
        } else {
          $message .= "Exten PIN\n";
          $sqlcmd = "UPDATE virt_exten SET passcode=? WHERE virt_exten=? AND company=?;";
          $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
          $sth->execute($pc,$vm,$section) or die "Cannot execute $sqlcmd!";
          $sth->finish();

# Same as DB.flag_change() in JS
          $dbh->do("UPDATE settings SET value=(EXTRACT(EPOCH FROM NOW())*1000)::bigint WHERE attrib='flag_change_date'");
        }
      }
    }
  }
  close VM;

#
# Get hold of the  flag_change_date and flag_updated_date values...
#
  my $fc_date=-1;
  my $fu_date=-1;
  my $isdn_type="hfc";
  my $dhcp_srv="off";
  my $ip="";
  my $mask="";
  my $gw="";
  my $dl_fw="";
  my $in_fw="";
  my $in_pf="";
  $dbh->do("START TRANSACTION") or die "Cannot start transaction!";
  my $sqlcmd = "SELECT * FROM settings;";
  my $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
  $sth->execute() or die "Cannot execute $sqlcmd!";
  while( my $row = $sth->fetchrow_hashref() ) {
    if( $row->{attrib} eq "flag_change_date" ) {
      $fc_date = $row->{value};
    }
    if( $row->{attrib} eq "flag_updated_date" ) {
      $fu_date = $row->{value};
    }
    if( $row->{attrib} eq "mail_delta" ) {
      $mail_delta = $row->{value} - 0;
    }
    if( $row->{attrib} eq "isdn_type" ) {
      $isdn_type = $row->{value};
    }
    if( $row->{attrib} eq "dhcp_srv" ) {
      $dhcp_srv = $row->{value};
    }
    if( $row->{attrib} eq "my_ip" ) {
      $ip = $row->{value};
    }
    if( $row->{attrib} eq "my_gw" ) {
      $gw = $row->{value};
    }
    if( $row->{attrib} eq "my_mask" ) {
      $mask = $row->{value};
    }
    if( $row->{attrib} eq "download_firmware" ) {
      $dl_fw = $row->{value};
    }
    if( $row->{attrib} eq "install_firmware" ) {
      $in_fw = $row->{value};
    }
    if( $row->{attrib} eq "install_tftpboot" ) {
      $in_pf = $row->{value};
    }
  }
  $dbh->do("COMMIT") or die "Cannot close transaction!";

  if( $fc_date != $fu_date ) {
    $message .= "\nA CHANGE WAS FLAGGED ($fc_date > $fu_date)!\n\n";
  } else {
    $dbh->disconnect();
    # STOP HERE AS NO CHANGE HAS OCCURRED!
    next;
  }

#
# Ensure that any FAX spool directories exist...
#
  $sqlcmd = "SELECT virt_exten FROM virt_exten WHERE ring_type='F';";
  $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
  $sth->execute() or die "Cannot execute $sqlcmd!";
  while( my $row = $sth->fetchrow_hashref() ) {
    if( ! -d "/var/spool/asterisk/fax/".$row->{virt_exten} ) {
      system("mkdir -p \"/var/spool/asterisk/fax/".$row->{virt_exten}."\"" );
      system("chmod 777 \"/var/spool/asterisk/fax/".$row->{virt_exten}."\"" );
    }
  }

#
# Ensure that any IVR directories exist...
#
  $sqlcmd = "SELECT DISTINCT company FROM phone;";
  $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
  $sth->execute() or die "Cannot execute $sqlcmd!";
  while( my $row = $sth->fetchrow_hashref() ) {
    if( ! -d "/var/spool/asterisk/ivr/".$row->{company} ) {
      system("mkdir -p \"/var/spool/asterisk/ivr/".$row->{company}."\"" );
      system("chmod 777 \"/var/spool/asterisk/ivr/".$row->{company}."\"" );
    }
    if( ! -d "/var/www/html/moh/" ) {
      system("mkdir -p \"/var/www/html/moh/\"" );
      system("chmod 777 \"/var/www/html/moh/\"" );
    }
    if( ! -d "/var/lib/asterisk/moh-native-".$row->{company} ) {
      system("mkdir -p \"/var/lib/asterisk/moh-native-".$row->{company}."\"" );
      system("chmod 777 \"/var/lib/asterisk/moh-native-".$row->{company}."\"" );
    }
    for( my $i=1; $i<6; $i++ ) {
      if( ! -d "/var/lib/asterisk/moh-native-".$row->{company}."-".$i ) {
        system("mkdir -p \"/var/lib/asterisk/moh-native-".$row->{company}."-".$i."\"" );
        system("chmod 777 \"/var/lib/asterisk/moh-native-".$row->{company}."-".$i."\"" );
      }
    }
  }

  $dbh->disconnect();

#
# Use 'wget' to fetch a page from the local webserver
# This causes the config files to be regenerated.
#
#  $res=`/usr/bin/wget --tries=1 -O - http://127.0.0.1/config/conf_files.whtm 2>/dev/null`;
  $res=`/var/whitebeam/bin/wbsh config/conf_files.whtm 2>/dev/null`;

  if( $res !~ /^OK/ ) {
    $message .= $res."\n";
    $message .= "Error running config page!\n";
    next;
  }

#
# Has iax.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/iax.conf $base/iax.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating iax.conf with the following changes:\n";
    $message .=  "=============================================\n";
    $message .=  "$diff\n";

    copy("$base/iax.conf","/etc/asterisk/iax.conf");
    system('/usr/sbin/asterisk -rx "iax2 reload"');
  }

#
# Has sip.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/sip.conf $base/sip.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating sip.conf with the following changes:\n";
    $message .=  "=============================================\n";
    $message .=  "$diff\n";

    copy("$base/sip.conf","/etc/asterisk/sip.conf");
    system('/usr/sbin/asterisk -rx "sip reload"');
  }

#
# Has features.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/features.conf $base/features.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating features.conf with the following changes:\n";
    $message .=  "==================================================\n";
    $message .=  "$diff\n";

    copy("$base/features.conf","/etc/asterisk/features.conf");
    system('/usr/sbin/asterisk -rx "reload res_features.so"');
  }

#
# Has extensions.conf changed?
#
  my $EXTENSIONS="no";
  $diff=`diff -Nu /etc/asterisk/extensions.conf $base/extensions.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating extensions.conf with the following changes:\n";
    $message .=  "====================================================\n";
    $message .=  "$diff\n";

    copy("$base/extensions.conf","/etc/asterisk/extensions.conf");
    $EXTENSIONS="yes";
  }

#
# Has meetme.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/meetme.conf $base/meetme.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating meetme.conf with the following changes:\n";
    $message .=  "================================================\n";
    $message .=  "$diff\n";

    copy("$base/meetme.conf","/etc/asterisk/meetme.conf.tmp");
    rename("/etc/asterisk/meetme.conf.tmp","/etc/asterisk/meetme.conf");
  }

#
# Has extensions-ivr.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/extensions-ivr.conf $base/extensions-ivr.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating extensions-ivr.conf with the following changes:\n";
    $message .=  "========================================================\n";
    $message .=  "$diff\n";

    copy("$base/extensions-ivr.conf","/etc/asterisk/extensions-ivr.conf");
    $EXTENSIONS="yes";
  }

#
# Has extensions-xlat.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/extensions-xlat.conf $base/extensions-xlat.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating extensions-xlat.conf with the following changes:\n";
    $message .=  "=========================================================\n";
    $message .=  "$diff\n";

    copy("$base/extensions-xlat.conf","/etc/asterisk/extensions-xlat.conf");
    $EXTENSIONS="yes";
  }

  if( $EXTENSIONS eq "yes" ) {
    system('/usr/sbin/asterisk -rx "extensions reload"');
  }

#
# Has queues.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/queues.conf $base/queues.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating queues.conf with the following changes:\n";
    $message .=  "================================================\n";
    $message .=  "$diff\n";

    copy("$base/queues.conf","/etc/asterisk/queues.conf");
    system('/usr/sbin/asterisk -rx "reload app_queue.so"');
  }

#
# Has astmanproxy.users changed?
#
  $diff=`diff -Nu /etc/asterisk/astmanproxy.users $base/astmanproxy.users`;
  if( length($diff) > 0 ) {
    $message .=  "Updating astmanproxy.users with the following changes:\n";
    $message .=  "======================================================\n";
    $message .=  "$diff\n";

    copy("$base/astmanproxy.users","/etc/asterisk/astmanproxy.users");
    system('killall -HUP astmanproxy');
  }

#
# Has musiconhold.conf changed?
#
  if( -f "$base/moh.conf" && ( ! -f "$base/musiconhold.conf"  || -M "$base/moh.conf" < -M "$base/musiconhold.conf" ) ) {
    rename "$base/moh.conf", "$base/musiconhold.conf";
    my $term=`grep -- '--END--' $base/musiconhold.conf`;
    if( length($term) != 10 ) {
      unlink "$base/musiconhold.conf";
    }
  }
  $diff=`diff -Nu /etc/asterisk/musiconhold.conf $base/musiconhold.conf`;
  if( -f $base."/musiconhold.conf" && length($diff) > 0 ) {
    $message .=  "Updating musiconhold.conf with the following changes:\n";
    $message .=  "======================================================\n";
    $message .=  "$diff\n";

    copy("$base/musiconhold.conf","/etc/asterisk/musiconhold.conf");
    system('/usr/sbin/asterisk -rx "moh reload"');
  }

#
# Has voicemail.conf changed?
#
  $diff=`diff -Nu /etc/asterisk/voicemail.conf $base/voicemail.conf`;
  if( length($diff) > 0 ) {
    $message .=  "Updating voicemail.conf with the following changes:\n";
    $message .=  "===================================================\n";
    $message .=  "$diff\n";

    copy("$base/voicemail.conf","$base/voicemail.conf.sent");
    copy("$base/voicemail.conf.sent","/etc/asterisk/voicemail.conf");
    system('/usr/sbin/asterisk -rx "reload app_voicemail.so"');
  } elsif( $count_updates > 0 ) {
    copy("$base/voicemail.conf","$base/voicemail.conf.sent");
  }

#
# SYSTEM FILES!!!
#

#
# /etc/hosts
#
  $diff=`diff -Nu /etc/hosts $base/etc_hosts 2>/dev/null`;
  if( -f "$base/etc_hosts" && length($diff) > 0 ) {
    $message .=  "Updating /etc/hosts with the following changes:\n";
    $message .=  "===============================================\n";
    $message .=  "$diff\n";
    $message .=  "Updating hostname with the following changes:\n";
    $message .=  "=============================================\n";
    $message .=  `diff -Nu /etc/hostname $base/etc_hostname`."\n";

    copy("$base/etc_hosts","/etc/hosts");
    copy("$base/etc_hostname","/etc/hostname");
    system('chmod 644 /etc/hosts /etc/hostname');

    # Yuck, perl calling perl....
    my $mlen = 0;
    my %map=(255=>8, 254=>7, 252=>6, 248=>5, 240=>4, 224=>3, 192=>2, 128=>1, 0=>0);
    for( my $i=0; $i<4; $i++ ) {
      $mlen += $map{(split /\./,$mask)[$i]}
    }
    system('perl -pi.bak -e"s/^(\s*kernel\s+.*ip=)\S+(.*)\$/\${1}'.${ip}.'\/'.${mlen}.'\${2}/; s/^(\s*kernel\s+.*gw=)\S+(.*)\$/\${1}'.${gw}.'\$2/; " /boot/grub/menu.lst');
    $message .=  "Updating grub/menu.lst with the following changes:\n";
    $message .=  "==================================================\n";
    $message .=  `diff -Nu /boot/grub/menu.lst.bak /boot/grub/menu.lst`."\n";
  }

#
# /etc/network/interfaces
#
  $diff=`diff -Nu /etc/network/interfaces $base/etc_network_interfaces 2>/dev/null`;
  if( -f "$base/etc_network_interfaces" && length($diff) > 0 ) {
    $message .=  "Updating /etc/network/interfaces with the following changes:\n";
    $message .=  "============================================================\n";
    $message .=  "$diff\n";

    copy("$base/etc_network_interfaces","/etc/network/interfaces");
    system('chown root:root /etc/network/interfaces');
    system('chmod 644 /etc/network/interfaces');
  }

#
# /etc/default/dhcp
#
  $diff=`diff -Nu /etc/default/dhcp $base/etc_default_dhcp 2>/dev/null`;
  if( -f "$base/etc_default_dhcp" && length($diff) > 0 ) {
    $message .=  "Updating /etc/default/dhcp with the following changes:\n";
    $message .=  "======================================================\n";
    $message .=  "$diff\n";

    copy("$base/etc_default_dhcp","/etc/default/dhcp");
    system('chown root:root /etc/default/dhcp');
    system('chmod 644 /etc/default/dhcp');
  }

#
# /etc/resolv.conf
#
  $diff=`diff -Nu /etc/resolv.conf $base/etc_resolv.conf 2>/dev/null`;
  if( -f "$base/etc_resolv.conf" && length($diff) > 0 ) {
    $message .=  "Updating /etc/resolv.conf with the following changes:\n";
    $message .=  "=====================================================\n";
    $message .=  "$diff\n";

    copy("$base/etc_resolv.conf","/etc/resolv.conf");
    system('chmod 644 /etc/resolv.conf');
  }

#
# /etc/ntp.conf
#
  $diff=`diff -Nu /etc/ntp.conf $base/etc_ntp.conf 2>/dev/null`;
  if( -f "$base/etc_ntp.conf" && length($diff) > 0 ) {
    $message .=  "Updating /etc/ntp.conf with the following changes:\n";
    $message .=  "==================================================\n";
    $message .=  "$diff\n";

    copy("$base/etc_ntp.conf","/etc/ntp.conf");
    system('chmod 644 /etc/ntp.conf');
  }

#
# /etc/aliases
#
  $diff=`diff -Nu /etc/aliases $base/etc_aliases 2>/dev/null`;
  if( -f "$base/etc_aliases" && length($diff) > 0 ) {
    $message .=  "Updating /etc/aliases with the following changes:\n";
    $message .=  "=================================================\n";
    $message .=  "$diff\n";

    copy("$base/etc_aliases","/etc/aliases");
    system('chmod 644 /etc/aliases');
    system('/usr/bin/newaliases');
  }

#
# op_buttons.cfg
#
  $diff=`diff -Nu /usr/local/fop/op_buttons.cfg $base/fop.conf 2>/dev/null`;
  if( -f "$base/fop.conf" && length($diff) > 0 ) {
    $message .=  "Updating /usr/local/fop/op_buttons.cfg with the following changes:\n";
    $message .=  "==================================================================\n";
    $message .=  "$diff\n";

    copy("$base/fop.conf","/usr/local/fop/op_buttons.cfg");
    system('chmod 644 /usr/local/fop/op_buttons.cfg');
    system('killall -HUP op_server.pl');
  }

#
# /configs/keys/*
#
  my $changedkeys=0;
  foreach my $i (glob $base."/keys/*.pub") {
    (my $target = $i) =~ s/^.*\/([^\/]+\.pub)$/\/var\/lib\/asterisk\/keys\/$1/;
    my $source = $i;
    $diff=`diff -Nu $target $source 2>/dev/null`;
    if( -f $source && length($diff) > 0 ) {
      $message .=  "Updating $target with the following changes:\n";
      $message .=  "=====================================".("=" x length($target))."\n";
      $message .=  "$diff\n";

      copy("$source","$target");
      system("chmod 644 $target");
      $changedkeys=1;
    }
  }
  if( $changedkeys == 1 ) {
    system('/usr/sbin/asterisk -rx "reload res_crypto.so"');
  }


#
# /tftpboot/*
#
  foreach my $i (glob $base."/tftp_*") {
    (my $target = $i) =~ s/^.*tftp_(.+)$/\/tftpboot\/$1/;
    my $source = $i;
    $diff=`diff -Nu $target $source 2>/dev/null`;
    if( -f $source && length($diff) > 0 ) {
      $message .=  "Updating $target with the following changes:\n";
      $message .=  "=====================================".("=" x length($target))."\n";
      $message .=  "$diff\n";

      copy("$source","$target");
      system("chmod 644 $target");
    }
  }

#
# /etc/bind/named.conf && /etc/bind/zonefile
#
  $diff=`diff -Nu /etc/bind/zonefile $base/named_zone 2>/dev/null`;
  if( length($diff) > 0 ) {
    $message .=  "Updating /etc/bind/named.conf with the following changes:\n";
    $message .=  "=========================================================\n";
    $message .=  `diff -Nu /etc/bind/named.conf $base/named_conf 2>/dev/null`."\n";

    $message .=  "Updating /etc/bind/zonefile with the following changes:\n";
    $message .=  "=======================================================\n";
    $message .=  "$diff\n";

    copy("$base/named_zone","/etc/bind/zonefile");
    system('chmod 644 /etc/bind/zonefile');

    copy("$base/named_conf","/etc/bind/named.conf");
    system('chmod 644 /etc/bind/named.conf');

    system('/etc/init.d/bind9 restart');
  }

#
# /etc/dhcpd.conf
#
  $diff=`diff -Nu /etc/dhcpd.conf $base/dhcpd_conf 2>/dev/null`;
  if( length($diff) > 0 ) {
    $message .=  "Updating /etc/dhcpd.conf with the following changes:\n";
    $message .=  "====================================================\n";
    $message .=  `diff -Nu /etc/dhcpd.conf $base/dhcpd_conf 2>/dev/null`."\n";

    copy("$base/dhcpd_conf","/etc/dhcpd.conf");
    system('chmod 644 /etc/dhcpd.conf');
    system('chown root:root /etc/dhcpd.conf');
  }

#
# sendmail smarthost
#
  if( -f "$base/sendmail_ds.txt" && -M "$base/sendmail_ds.txt" < -M "/etc/mail/sendmail.cf" ) {
    $message .=  "Updating /etc/mail/sendmail.cf\n";
    $message .=  "==============================\n";

    # Yuck, perl calling perl....
    system('perl -pi.bak -e"s/^Dj.*?\n//m;s/^DM.*?\n//m;s/^DS.*/`cat /var/www/html/config/configs/sendmail_ds.txt`/" /etc/mail/sendmail.cf');
    system('killall -HUP sendmail-mta');
  }

#
# zapata.conf
#
  if( -f "$base/zapata.conf" ) {
  $diff=`diff -Nu /etc/asterisk/zapata.conf $base/zapata.conf 2>/dev/null`;
    if( length($diff) > 0 ) {
      $message .=  "Updating /etc/asterisk/zapata.conf with the following changes:\n";
      $message .=  "==============================================================\n";
      $message .=  $diff."\n";

      copy("$base/zapata.conf","/etc/asterisk/zapata.conf");
      system('chmod 644 /etc/asterisk/zapata.conf');
      system('chown root:root /etc/asterisk/zapata.conf');
    }
  }

#
# DHCPD on or off
#
  if( defined(glob "/etc/rc*.d/S*dhcp") && $dhcp_srv ne "on" ) {
    $message .= "Disabling DHCPD\n";
    $message .= "===============\n";
    system("/usr/sbin/update-rc.d -f dhcp remove");

  }
  if( !defined(glob "/etc/rc*.d/S*dhcp") && $dhcp_srv eq "on" ) {
    $message .= "Enabling DHCPD\n";
    $message .= "==============\n";
    system("/usr/sbin/update-rc.d dhcp defaults");
  }

  $dbh = DBI->connect( "dbi:Pg:dbname=$db;host=127.0.0.1", "$user", "$pass" ) or die "Cannot connect to database!";
  my $last_req=undef;
  my $last_cmp=undef;
  my $command=undef;

  $sqlcmd = "UPDATE settings SET value='' WHERE attrib IN ('download_firmware','install_firmware','install_tftpboot');";
  if( defined($dl_fw) && $dl_fw ne "" ) {
    system("/usr/local/bin/dl_fw.pl",$dl_fw);
    $dbh->do($sqlcmd);
  } elsif( defined($in_fw) && $in_fw ne "" ) {
    system("/usr/local/bin/in_fw.pl",$in_fw);
    $dbh->do($sqlcmd);
  } elsif( defined($in_pf) && $in_pf ne "" ) {
    system("/var/www/html/cgi-bin/firmware_stat.pl","--confirm","$in_pf");
    $dbh->do($sqlcmd);
  }

  $sqlcmd = "SELECT * FROM settings;";
  $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";
  $sth->execute() or die "Cannot execute $sqlcmd!";
  while( my $row = $sth->fetchrow_hashref() ) {
    if( $row->{attrib} eq "last-action-req" ) {
      $last_req = $row->{value};
    }
    if( $row->{attrib} eq "last-action-cmp" ) {
      $last_cmp = $row->{value};
    }
  }

  $sqlcmd=undef;
  if( defined($last_req) && defined($last_cmp) ) {
    if( $last_req =~ /^HALT/ && $last_req ne $last_cmp ) {
      $message .=  "Got request for: HALT\n";
      $command="/usr/local/bin/zap_config; /sbin/halt -p";
      $sqlcmd="UPDATE settings SET value='".$last_req."' WHERE attrib='last-action-cmp';";
    } elsif ( $last_req =~ /^REBOOT/ && $last_req ne $last_cmp ) {
      $message .=  "Got request for: REBOOT\n";
      $command="/usr/local/bin/zap_config; /sbin/reboot";
      $sqlcmd="UPDATE settings SET value='".$last_req."' WHERE attrib='last-action-cmp';";
    } elsif ( $last_req =~ /^RESTART/ && $last_req ne $last_cmp ) {
      $message .=  "Got request for: RESTART\n";
      $command="/sbin/service asterisk stop; /usr/local/bin/zap_config; /sbin/service zaptel restart; /sbin/service asterisk start";
      $sqlcmd="UPDATE settings SET value='".$last_req."' WHERE attrib='last-action-cmp';";
    }
  }
  if( defined($command) && defined($sqlcmd) ) {
    $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
    $sth->execute() or die "Cannot execute $sqlcmd!";
    $sqlcmd="UPDATE settings SET value='None' WHERE attrib='last-action-req'";
    $sth = $dbh->prepare($sqlcmd) or die "Cannot prepare $sqlcmd!";;
    $sth->execute() or die "Cannot execute $sqlcmd!";
    system($command);
  }

  if( $fu_date == -1 ) {
    $dbh->do("INSERT INTO settings(attrib,value) VALUES('flag_updated_date',1)");
  }
  $dbh->do("UPDATE settings SET value=".$fc_date."::bigint WHERE attrib='flag_updated_date' AND value < ".$fc_date."::bigint");

  $dbh->disconnect();
  $message .= "Done.\n";
}
