  var days=['Sun','Mon','Tue','Wed','Thu','Fri','Sat'];
  var short_days=['Su','Mo','Tu','We','Th','Fr','Sa'];
  var rate_type={P:'Peak', O:'Off-Peak', W:'Weekend', X:'Other', H:'Hidden'};
  var qual_type={F:'Fully qualified',S:'Short form',I:'Internal'};
  var months=['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];

  function parse_date(ds) {
    if( ds == null || ds.length == 0 )
      return null;
    var d = ds.match(/^(|.+ )([0-9]?[0-9])(| .+)$/);
    var m = ds.toLowerCase().match(/^(|[^a-z]+ )([a-z]{3})(| [^a-z]+)$/);
    var y = ds.match(/^(|.+ )([0-9]{4})(| .+)$/);
    var t = ds.match(/^(|.+ )([0-2]?[0-9]:[0-5][0-9])(| .+)$/);

    if( d != null )
      d = d[2];
    if( m != null ) {
      m = m[2];
      var m1="janfebmaraprmayjunjulaugsepoctnovdec".search(m);
      if( m1 < 0 )
        m=null;
      else
        m = m1 / 3;
    }
    if( y != null )
      y = y[2];
    if( t != null ) {
      t = t[2];
      t1 = t.split(":")[0] - 0;
      t2 = t.split(":")[1] - 0;
      if( t1 > 23 )
        t=null;
    }
    if( t==null || d==null || m==null || y==null )
      return null;

    var dd = new Date(0);
    dd.setFullYear(y-0);
    dd.setMonth(m-0);
    dd.setDate(d-0);
    dd.setHours(t1);
    dd.setMinutes(t2);
    return dd;
  }

  function peak_rate_parse(r) {
    res=[];
    var r1 = r.split(",");
    for( var i=0; i<r1.length; i++ ) {
      if( r1[i] == 'default' )
        r1[i]='Su-Sa==O';
      var r2 = r1[i].split("=");
      if( r2.length != 3 )
        continue;
// Parse day range
      var d1="sumotuwethfrsa".search(r2[0].slice(0,2).toLowerCase());
      if( d1 < 0 )
        continue;
      d1 = d1 / 2
      var d2="-su-mo-tu-we-th-fr-sa".search(r2[0].slice(2,5).toLowerCase());
      if( r2[0].slice(2,3) != '-' )
        d2 = d1 * 3;
      if( d2 < 0 )
        continue;
      d2 = d2 / 3
// Parse hour range (as seconds)
      if( r2[1] == "" ) r2[1]="00:00-23:59";
      var hh = r2[1].match(/^([0-2]?[0-9]):([0-5][0-9])-([0-2]?[0-9]):([0-5][0-9])$/)
      if( hh==null || hh.length != 5 || hh[1] > 23 || hh[3] > 23 )
        continue;
      var h1=hh[1]*3600 + hh[2]*60;
      var h2=hh[3]*3600 + hh[4]*60 + 59;
      if( h1 >= h2 )
        continue;

// Convert hours back from seconds to formatted strings for printing.
      h1s="";
      if( h1/3600 < 10 ) h1s += "0";
      h1s += Math.floor(h1/3600) + ":";
      if( (h1%3600)/60 < 10 ) h1s += "0";
      h1s += Math.floor((h1%3600)/60);
      h2s="";
      if( h2/3600 < 10 ) h2s += "0";
      h2s += Math.floor(h2/3600) + ":";
      if( (h2%3600)/60 < 10 ) h2s += "0";
      h2s += Math.floor((h2%3600)/60);
      res.push({fromday:d1, today:d2, fromhour:h1, tohour:h2, fromhour_s:h1s, tohour_s:h2s, rate:r2[2]});
    }
    return res;
  }

  function peak_rate_rebuild(ra) {
    res='';
    if( ra.length == 1 && ra[0].fromday==0 && ra[0].today==6 && ra[0].fromhour==0 && ra[0].tohour==86399 && ra[0].rate=="O" )
      res = 'default';
    else {
      for( var i=0; i<ra.length; i++ ) {
        var tmp='';
        tmp+=short_days[ra[i].fromday];
        if( ra[i].fromday != ra[i].today )
          tmp+="-"+short_days[ra[i].today];
        tmp+="=";
        if( ra[i].fromhour!=0 || ra[i].tohour!=86399 )
          tmp+=ra[i].fromhour_s+"-"+ra[i].tohour_s;
        tmp+="="+ra[i].rate;
        // Ensure no dupes.
        if( (","+res+",").search(tmp) == -1 ) {
          if( i > 0 )
            res+=",";
          res += tmp;
        }
      }
    }
    if( res=='' )
      res='default';
    return res;
  }

  function get_rates() {
    var rates=[];
    if ((pgconn=DB.open())) {
      rb.debug.write("DATABASE open OK<br>");
      pgres = pgconn.exec('SELECT rate.*, rate_res.company, rate_res.provider, count(rate_pfx.name) AS prefixes'+
                          ' FROM rate'+
                          ' LEFT JOIN rate_res ON rate.name = rate_res.rate_name'+
                          ' LEFT JOIN rate_pfx ON rate.name = rate_pfx.name'+
                          ' GROUP BY rate.name, peakdef, fromdate, todate, rate_res.company, rate_res.provider'+
                          ' ORDER BY fromdate DESC, todate DESC, name');
      if( pgres!=null && pgres.success ) {
        for( var r=0;r<pgres.numRows; r++ ) {
          var row = pgres.fetchRowObject(r);

          var dd = row.fromdate.split(/[^\d]/);
          var d = new Date(0);
          d.setUTCFullYear(dd[0]);
          d.setUTCMonth(dd[1]-1);
          d.setUTCDate(dd[2]);
          d.setUTCHours(dd[3]);
          d.setUTCMinutes(dd[4]);
          d.setUTCSeconds(dd[5]);
          dd = row.fromdate.match(/^.*((?:\+|-)\d+)$/);
          if( dd != null && dd[1] != null )
            d=new Date( d - dd[1]*3600000);
          row.fromdate = d;

          dd = row.todate.split(/[^\d]/);
          d = new Date(0);
          d.setUTCFullYear(dd[0]);
          d.setUTCMonth(dd[1]-1);
          d.setUTCDate(dd[2]);
          d.setUTCHours(dd[3]);
          d.setUTCMinutes(dd[4]);
          d.setUTCSeconds(dd[5]);
          dd = row.todate.match(/^.*((?:\+|-)\d+)$/);
          if( dd != null && dd[1] != null )
            d=new Date( d - dd[1]*3600000);
          row.todate = d;

          row.peak_array=peak_rate_parse(row.peakdef);

          var rr={};
          for( var x in row )
            rr[x]=row[x];
          rates.push(rr);
        }
      }
    }
    return rates;
  }
