--
-- PostgreSQL database dump
--

SET client_encoding = 'SQL_ASCII';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

--
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'Standard public schema';


--
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: -; Owner: postgres
--

CREATE PROCEDURAL LANGUAGE plpgsql;


SET search_path = public, pg_catalog;

--
-- Name: matchstring(text, text); Type: FUNCTION; Schema: public; Owner: whitebeam
--

CREATE FUNCTION matchstring(searchstr text, srcstr text) RETURNS integer
    AS $$
DECLARE
        count integer := 0;
        keywords text[] := string_to_array(trim(lower(searchStr)),' ');
        str text := ' ' || lower(srcStr) || ' ';
BEGIN
        IF str<>'' THEN
                FOR idx IN 1..array_upper(keywords,1) LOOP
                        IF str LIKE '%'||keywords[idx]||'%'
                        THEN
                                count := count+1;
                        END IF;
                END LOOP;
        END IF;
        RETURN count;
END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION public.matchstring(searchstr text, srcstr text) OWNER TO whitebeam;

--
-- Name: plpgsql_call_handler(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


ALTER FUNCTION public.plpgsql_call_handler() OWNER TO postgres;

--
-- Name: collection_id_seq; Type: SEQUENCE; Schema: public; Owner: whitebeam
--

CREATE SEQUENCE collection_id_seq
    START WITH 100
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 50;


ALTER TABLE public.collection_id_seq OWNER TO whitebeam;

SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: collection_types; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE collection_types (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    collection_id bigint DEFAULT nextval(('collection_id_seq'::text)::regclass) NOT NULL,
    name character varying(50) NOT NULL,
    opaque_data character varying
);


ALTER TABLE public.collection_types OWNER TO whitebeam;

--
-- Name: collections; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE collections (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    collection_id bigint NOT NULL,
    left_type character(1) NOT NULL,
    left_id bigint NOT NULL,
    right_type character(1) NOT NULL,
    right_id bigint NOT NULL,
    sub_type character varying(30),
    custom_data character varying,
    recorded_time timestamp without time zone
);


ALTER TABLE public.collections OWNER TO whitebeam;

--
-- Name: contact_att; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE contact_att (
    instance character(1) NOT NULL,
    client_id character varying(50) NOT NULL,
    contact_id bigint NOT NULL,
    community_id bigint NOT NULL,
    inherited smallint
);


ALTER TABLE public.contact_att OWNER TO whitebeam;

--
-- Name: contact_item; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE contact_item (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    contact_id bigint NOT NULL,
    item_id bigint NOT NULL
);


ALTER TABLE public.contact_item OWNER TO whitebeam;

--
-- Name: contact_node; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE contact_node (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    contact_id bigint NOT NULL,
    node_id bigint NOT NULL
);


ALTER TABLE public.contact_node OWNER TO whitebeam;

--
-- Name: contacts; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE contacts (
    instance character(1) NOT NULL,
    client_id character varying(50) NOT NULL,
    contact_id bigint DEFAULT nextval(('contacts_contact_id_seq'::text)::regclass) NOT NULL,
    uname character varying(32) NOT NULL,
    "type" character varying(20) DEFAULT 'INDIVIDUAL'::character varying NOT NULL,
    parent bigint,
    name character varying(240),
    phone character varying(32),
    fax character varying(32),
    email character varying(240),
    basic_pw character varying(128),
    description character varying(1000),
    custom_data character varying(8192),
    value bigint DEFAULT 0 NOT NULL,
    keywords character varying(1000)
);


ALTER TABLE public.contacts OWNER TO whitebeam;

--
-- Name: contacts_contact_id_seq; Type: SEQUENCE; Schema: public; Owner: whitebeam
--

CREATE SEQUENCE contacts_contact_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 50;


ALTER TABLE public.contacts_contact_id_seq OWNER TO whitebeam;

--
-- Name: files; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE files (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    file_id bigint DEFAULT nextval(('files_file_id_seq'::text)::regclass) NOT NULL,
    file_name character varying(50) NOT NULL,
    description character varying(250),
    file_size bigint NOT NULL,
    mime character varying(200),
    "location" character varying(250),
    last_modified_date date NOT NULL,
    "owner" character varying(50),
    data_new bytea
);


ALTER TABLE public.files OWNER TO whitebeam;

--
-- Name: files_file_id_seq; Type: SEQUENCE; Schema: public; Owner: whitebeam
--

CREATE SEQUENCE files_file_id_seq
    START WITH 100
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 50;


ALTER TABLE public.files_file_id_seq OWNER TO whitebeam;

--
-- Name: item_files; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE item_files (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    item_id bigint NOT NULL,
    file_id bigint NOT NULL,
    file_order integer NOT NULL
);


ALTER TABLE public.item_files OWNER TO whitebeam;

--
-- Name: item_leafs; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE item_leafs (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    node_id bigint NOT NULL,
    item_id bigint NOT NULL
);


ALTER TABLE public.item_leafs OWNER TO whitebeam;

--
-- Name: item_prices; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE item_prices (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    item_id bigint NOT NULL,
    price_order integer NOT NULL,
    price bigint NOT NULL,
    currency character varying(20) DEFAULT 'GBP'::character varying NOT NULL,
    description character varying(100),
    last_modified_date date NOT NULL,
    pricedata character varying(1000)
);


ALTER TABLE public.item_prices OWNER TO whitebeam;

--
-- Name: itemdata; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE itemdata (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    item_id bigint NOT NULL,
    itemdata character varying(7000)
);


ALTER TABLE public.itemdata OWNER TO whitebeam;

--
-- Name: items; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE items (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    item_id bigint DEFAULT nextval(('items_item_id_seq'::text)::regclass) NOT NULL,
    client_item_code character varying(250) NOT NULL,
    description character varying(250) NOT NULL,
    container_mkr integer NOT NULL,
    container_id bigint,
    search_keywords character varying(250),
    number_of_visits bigint,
    last_modified_date timestamp without time zone
);


ALTER TABLE public.items OWNER TO whitebeam;

--
-- Name: items_item_id_seq; Type: SEQUENCE; Schema: public; Owner: whitebeam
--

CREATE SEQUENCE items_item_id_seq
    START WITH 100
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 50;


ALTER TABLE public.items_item_id_seq OWNER TO whitebeam;

--
-- Name: node_leafs; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE node_leafs (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    node_id bigint NOT NULL,
    parent_node bigint NOT NULL
);


ALTER TABLE public.node_leafs OWNER TO whitebeam;

--
-- Name: searchtree; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE searchtree (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    node_id bigint DEFAULT nextval(('searchtree_node_id_seq'::text)::regclass) NOT NULL,
    type_id character varying(2) NOT NULL,
    name character varying(250) NOT NULL,
    description character varying(1000),
    parent_node bigint,
    file_id bigint,
    nodedata character varying(2000)
);


ALTER TABLE public.searchtree OWNER TO whitebeam;

--
-- Name: searchtree_node_id_seq; Type: SEQUENCE; Schema: public; Owner: whitebeam
--

CREATE SEQUENCE searchtree_node_id_seq
    START WITH 100
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 50;


ALTER TABLE public.searchtree_node_id_seq OWNER TO whitebeam;

--
-- Name: stockdata; Type: TABLE; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE TABLE stockdata (
    instance character(1) NOT NULL,
    client_id character varying(30) NOT NULL,
    item_id bigint NOT NULL,
    stockdata character varying(7000)
);


ALTER TABLE public.stockdata OWNER TO whitebeam;

--
-- Name: ca_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY contact_att
    ADD CONSTRAINT ca_pk PRIMARY KEY (instance, client_id, contact_id, community_id);


--
-- Name: con_collection_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY collections
    ADD CONSTRAINT con_collection_pk PRIMARY KEY (instance, client_id, collection_id, left_type, left_id, right_type, right_id);


--
-- Name: con_collection_type_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY collection_types
    ADD CONSTRAINT con_collection_type_pk PRIMARY KEY (instance, client_id, collection_id, name);


--
-- Name: con_collection_type_unique; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY collection_types
    ADD CONSTRAINT con_collection_type_unique UNIQUE (instance, client_id, name);


--
-- Name: con_collection_type_unique2; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY collection_types
    ADD CONSTRAINT con_collection_type_unique2 UNIQUE (instance, client_id, collection_id);


--
-- Name: con_it_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY contact_item
    ADD CONSTRAINT con_it_pk PRIMARY KEY (instance, client_id, contact_id, item_id);


--
-- Name: con_no_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY contact_node
    ADD CONSTRAINT con_no_pk PRIMARY KEY (instance, client_id, contact_id, node_id);


--
-- Name: cos_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY contacts
    ADD CONSTRAINT cos_pk PRIMARY KEY (instance, client_id, contact_id);


--
-- Name: files_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY files
    ADD CONSTRAINT files_pk PRIMARY KEY (instance, client_id, file_id);


--
-- Name: item_files_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY item_files
    ADD CONSTRAINT item_files_pk PRIMARY KEY (client_id, instance, item_id, file_order);


--
-- Name: item_leaf_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY item_leafs
    ADD CONSTRAINT item_leaf_pk PRIMARY KEY (instance, client_id, node_id, item_id);


--
-- Name: item_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY items
    ADD CONSTRAINT item_pk PRIMARY KEY (instance, client_id, item_id);


--
-- Name: itemdata_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY itemdata
    ADD CONSTRAINT itemdata_pk PRIMARY KEY (instance, client_id, item_id);


--
-- Name: node_leafs_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY node_leafs
    ADD CONSTRAINT node_leafs_pk PRIMARY KEY (instance, client_id, node_id, parent_node);


--
-- Name: price_item_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY item_prices
    ADD CONSTRAINT price_item_pk PRIMARY KEY (instance, client_id, item_id, price_order);


--
-- Name: searchtree_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY searchtree
    ADD CONSTRAINT searchtree_pk PRIMARY KEY (instance, client_id, node_id);


--
-- Name: stock_pk; Type: CONSTRAINT; Schema: public; Owner: whitebeam; Tablespace: 
--

ALTER TABLE ONLY stockdata
    ADD CONSTRAINT stock_pk PRIMARY KEY (instance, client_id, item_id);


--
-- Name: cat_item_index; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX cat_item_index ON items USING btree (instance, client_id, client_item_code);


--
-- Name: cat_node_index; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX cat_node_index ON searchtree USING btree (instance, client_id, name);


--
-- Name: co_pa_ind; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX co_pa_ind ON contacts USING btree (instance, client_id, parent);


--
-- Name: co_ty_ind; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX co_ty_ind ON contacts USING btree (instance, client_id, "type");


--
-- Name: collection_left; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX collection_left ON collections USING btree (instance, client_id, left_type, left_id);


--
-- Name: collection_right; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX collection_right ON collections USING btree (instance, client_id, right_type, right_id);


--
-- Name: cos_uk; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE UNIQUE INDEX cos_uk ON contacts USING btree (instance, client_id, uname);


--
-- Name: file_name_index; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX file_name_index ON files USING btree (instance, client_id, file_name);


--
-- Name: it_co_ind; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX it_co_ind ON items USING btree (instance, client_id, container_mkr);


--
-- Name: nl_par_uk; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX nl_par_uk ON node_leafs USING btree (parent_node);


--
-- Name: se_pa_ind; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX se_pa_ind ON searchtree USING btree (parent_node);


--
-- Name: se_ty_ind; Type: INDEX; Schema: public; Owner: whitebeam; Tablespace: 
--

CREATE INDEX se_ty_ind ON searchtree USING btree (instance, client_id, type_id);


--
-- Name: clean_collections; Type: RULE; Schema: public; Owner: whitebeam
--

CREATE RULE clean_collections AS ON DELETE TO collection_types DO DELETE FROM collections WHERE (((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (old.collection_id = collections.collection_id));


--
-- Name: clean_collections_c; Type: RULE; Schema: public; Owner: whitebeam
--

CREATE RULE clean_collections_c AS ON DELETE TO contacts DO DELETE FROM collections WHERE (((((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (collections.left_type = 'C'::bpchar)) AND (old.contact_id = collections.left_id)) OR ((((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (collections.right_type = 'C'::bpchar)) AND (old.contact_id = collections.right_id)));


--
-- Name: clean_collections_i; Type: RULE; Schema: public; Owner: whitebeam
--

CREATE RULE clean_collections_i AS ON DELETE TO items DO DELETE FROM collections WHERE (((((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (collections.left_type = 'I'::bpchar)) AND (old.item_id = collections.left_id)) OR ((((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (collections.right_type = 'I'::bpchar)) AND (old.item_id = collections.right_id)));


--
-- Name: clean_collections_n; Type: RULE; Schema: public; Owner: whitebeam
--

CREATE RULE clean_collections_n AS ON DELETE TO searchtree DO DELETE FROM collections WHERE (((((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (collections.left_type = 'I'::bpchar)) AND (old.node_id = collections.left_id)) OR ((((old.instance = collections.instance) AND ((old.client_id)::text = (collections.client_id)::text)) AND (collections.right_type = 'I'::bpchar)) AND (old.node_id = collections.right_id)));


--
-- Name: ca_cos_comm; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY contact_att
    ADD CONSTRAINT ca_cos_comm FOREIGN KEY (instance, client_id, community_id) REFERENCES contacts(instance, client_id, contact_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: ca_cos_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY contact_att
    ADD CONSTRAINT ca_cos_fk FOREIGN KEY (instance, client_id, contact_id) REFERENCES contacts(instance, client_id, contact_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: ib_items_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY item_files
    ADD CONSTRAINT ib_items_fk FOREIGN KEY (instance, client_id, item_id) REFERENCES items(instance, client_id, item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: it_items_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY itemdata
    ADD CONSTRAINT it_items_fk FOREIGN KEY (instance, client_id, item_id) REFERENCES items(instance, client_id, item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: item_leaf_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY item_leafs
    ADD CONSTRAINT item_leaf_fk FOREIGN KEY (instance, client_id, item_id) REFERENCES items(instance, client_id, item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: item_node_tree_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY item_leafs
    ADD CONSTRAINT item_node_tree_fk FOREIGN KEY (instance, client_id, node_id) REFERENCES searchtree(instance, client_id, node_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: nl_se_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY node_leafs
    ADD CONSTRAINT nl_se_fk FOREIGN KEY (instance, client_id, node_id) REFERENCES searchtree(instance, client_id, node_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: price_item_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY item_prices
    ADD CONSTRAINT price_item_fk FOREIGN KEY (instance, client_id, item_id) REFERENCES items(instance, client_id, item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: stock_items_fk; Type: FK CONSTRAINT; Schema: public; Owner: whitebeam
--

ALTER TABLE ONLY stockdata
    ADD CONSTRAINT stock_items_fk FOREIGN KEY (instance, client_id, item_id) REFERENCES items(instance, client_id, item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

